unit AnsiStringVisualizer;

interface

procedure Register;

implementation

uses
  Classes, Forms, SysUtils, ToolsAPI;

resourcestring
  sAnsiStringVisualizerName = 'AnsiString/UTF8String Visualizer for C++';
  sAnsiStringVisualizerDescription = 'Displays the actual string value for AnsiString and UTF8String instances';

type
  TAnsiStringVisualizer = class(TInterfacedObject, IOTADebuggerVisualizer,
    IOTADebuggerVisualizerValueReplacer, IOTAThreadNotifier)
  private
    FNotifierIndex: Integer;
    FCompleted: Boolean;
    FDeferredResult: string;
  public
    { IOTADebuggerVisualizer }
    function GetSupportedTypeCount: Integer;
    procedure GetSupportedType(Index: Integer; var TypeName: string;
      var AllDescendants: Boolean);
    function GetVisualizerIdentifier: string;
    function GetVisualizerName: string;
    function GetVisualizerDescription: string;
    { IOTADebuggerVisualizerValueReplacer }
    function GetReplacementValue(const Expression, TypeName, EvalResult: string): string;
    { IOTAThreadNotifier }
    procedure EvaluteComplete(const ExprStr: string; const ResultStr: string;
      CanModify: Boolean; ResultAddress: Cardinal; ResultSize: Cardinal;
      ReturnCode: Integer);
    procedure ModifyComplete(const ExprStr: string; const ResultStr: string;
      ReturnCode: Integer);
    procedure ThreadNotify(Reason: TOTANotifyReason);
    procedure AfterSave;
    procedure BeforeSave;
    procedure Destroyed;
    procedure Modified;
  end;


const
  AnsiStringVisualizerTypes: array[0..1] of string =
  (
    'System::AnsiStringT<0>',
    'System::AnsiStringT<65001>'
  );

{ TAnsiStringVisualizer }

procedure TAnsiStringVisualizer.AfterSave;
begin
  // don't care about this notification
end;

procedure TAnsiStringVisualizer.BeforeSave;
begin
  // don't care about this notification
end;

procedure TAnsiStringVisualizer.Destroyed;
begin
  // don't care about this notification
end;

procedure TAnsiStringVisualizer.Modified;
begin
  // don't care about this notification
end;

procedure TAnsiStringVisualizer.ModifyComplete(const ExprStr,
  ResultStr: string; ReturnCode: Integer);
begin
  // don't care about this notification
end;

procedure TAnsiStringVisualizer.EvaluteComplete(const ExprStr,
  ResultStr: string; CanModify: Boolean; ResultAddress, ResultSize: Cardinal;
  ReturnCode: Integer);
begin
  FCompleted := True;
  if ReturnCode = 0 then
    FDeferredResult := ResultStr;
end;

procedure TAnsiStringVisualizer.ThreadNotify(Reason: TOTANotifyReason);
begin
  // don't care about this notification
end;

function TAnsiStringVisualizer.GetReplacementValue(
  const Expression, TypeName, EvalResult: string): string;
var
  CurProcess: IOTAProcess;
  CurThread: IOTAThread;
  ResultStr: array[0..4095] of Char;
  CanModify: Boolean;
  ResultAddr, ResultSize, ResultVal: LongWord;
  EvalRes: TOTAEvaluateResult;
  DebugSvcs: IOTADebuggerServices;
begin
  begin
    Result := EvalResult;
    if Supports(BorlandIDEServices, IOTADebuggerServices, DebugSvcs) then
      CurProcess := DebugSvcs.CurrentProcess;
    if CurProcess <> nil then
    begin
      CurThread := CurProcess.CurrentThread;
      if CurThread <> nil then
      begin
        EvalRes := CurThread.Evaluate(Expression+'.Data', @ResultStr, Length(ResultStr),
          CanModify, eseAll, '', ResultAddr, ResultSize, ResultVal, '', 0);
        case EvalRes of
          erOK: Result := ResultStr;
          erDeferred:
            begin
              FCompleted := False;
              FDeferredResult := '';
              FNotifierIndex := CurThread.AddNotifier(Self);
              while not FCompleted do
                DebugSvcs.ProcessDebugEvents;
              CurThread.RemoveNotifier(FNotifierIndex);
              FNotifierIndex := -1;
              if FDeferredResult <> '' then
                Result := FDeferredResult
              else
                Result := EvalResult;
            end;
          erBusy:
            begin
              DebugSvcs.ProcessDebugEvents;
              Result := GetReplacementValue(Expression, TypeName, EvalResult);
            end;
        end;
      end;
    end;
  end;
end;

function TAnsiStringVisualizer.GetSupportedTypeCount: Integer;
begin
  Result := Length(AnsiStringVisualizerTypes);
end;

procedure TAnsiStringVisualizer.GetSupportedType(Index: Integer; var TypeName: string;
  var AllDescendants: Boolean);
begin
  AllDescendants := False;
  TypeName := AnsiStringVisualizerTypes[Index];
end;

function TAnsiStringVisualizer.GetVisualizerDescription: string;
begin
  Result := sAnsiStringVisualizerDescription;
end;

function TAnsiStringVisualizer.GetVisualizerIdentifier: string;
begin
  Result := ClassName;
end;

function TAnsiStringVisualizer.GetVisualizerName: string;
begin
  Result := sAnsiStringVisualizerName;
end;

var
  AnsiStringVis: IOTADebuggerVisualizer;

procedure Register;
begin
  AnsiStringVis := TAnsiStringVisualizer.Create;
  (BorlandIDEServices as IOTADebuggerServices).RegisterDebugVisualizer(AnsiStringVis);
end;

procedure RemoveVisualizer;
var
  DebuggerServices: IOTADebuggerServices;
begin
  if Supports(BorlandIDEServices, IOTADebuggerServices, DebuggerServices) then
  begin
    DebuggerServices.UnregisterDebugVisualizer(AnsiStringVis);
    AnsiStringVis := nil;
  end;
end;

initialization
finalization
  RemoveVisualizer;
end.

